@@ -1,30 +1,74 @@
 function ClearClickHandler(ns) {
   this.ns = ns;
-  if (ns.geckoVersionCheck("1.9.2") < 0)
+  if (ns.geckoVersionCheck("1.9.2") < 0) {
     INCLUDE("ClearClickHandlerLegacy");
+  }
 }
 
 ClearClickHandler.prototype = {
   
-  // TODO:
-  // 1. try to use MozAfterPaint (Fx 3.1) to intercept "Sudden Reveal" attacks
-  // 2. try to use http://www.oxymoronical.com/experiments/apidocs/interface/nsIDOMWindowUtils:compareCanvases
-  
   uiEvents: ["mousedown", "mouseup", "click", "dblclick", "keydown", "keypress", "keyup", "blur"],
+  
+  rapidFire: {
+    quarantine: 800,
+    topSite: null,
+    ts: 0,
+    mouse: false,
+    screenX: -16,
+    screenY: -16,
+    
+    check: function(ev, topSite, ts) {
+      const typeChar = ev.type[0];
+      
+      let mouse = false;
+      switch(typeChar) {
+        case 'c': case 'm':
+          mouse = true;
+        case 'k':
+        break;
+      
+        default:
+          return false;
+      }
+      
+      if (this.mouse === mouse) {
+        if (topSite !== this.topSite &&
+            (ts - this.ts) < this.quarantine &&
+            (!mouse || Math.sqrt(Math.pow(ev.screenX - this.screenX, 2) + Math.pow(ev.screenY - this.screenY, 2)) < 16)) {
+          this.ts = ts - this.quarantine / 2;
+          return ns.getPref("clearClick.rapidFireCheck");
+        }
+      } else this.mouse = mouse;
+      
+      if (mouse) {
+        this.screenX = ev.screenX;
+        this.screenY = ev.screenY;
+      }
+      
+      this.topSite = topSite;
+      this.ts = ts;
+      
+      return false;
+  
+    }
+  },
+  
   install: function(browser) {
+    
+    var doc = browser.ownerDocument;
+    if (!("__ClearClick__" in doc)) {
+      doc.__ClearClick__ = true;
+      for each(let et in ["keydown", "mousedown"]) doc.addEventListener(et, this, true);
+    }
+    
     var ceh = browser.docShell.chromeEventHandler;
     var l = this._listener;
-    for each(var et in this.uiEvents) ceh.addEventListener(et, this._listener, true);
+    for each(var et in this.uiEvents) ceh.addEventListener(et, this, true);
   },
+ 
   
   exceptions: null,
   
-  get _listener() {
-    var self = this;
-    var l = function(ev) { self.handle(ev); };
-    this.__defineGetter__("_listener", function() { return l; });
-  },
-  
   sameSiteParents: function(w) {
     const ns = this.ns;
     var site = ns.getSite(w.location.href);
@@ -45,7 +89,7 @@ ClearClickHandler.prototype = {
   appliesHere: function(url) {
     const ns = this.ns;
     return ns.appliesHere(ns.clearClick, url) &&
-     !(this.exceptions && this.exceptions.test(url) && ns.isJSEnabled(ns.getSite(url)));
+      !(this.exceptions && this.exceptions.test(url));
   },
   
   checkSubexception: function(url) {
@@ -90,7 +134,7 @@ ClearClickHandler.prototype = {
   },
   
   isEmbed: function(o) {
-    return (o instanceof CI.nsIDOMHTMLObjectElement || o instanceof CI.nsIDOMHTMLEmbedElement)
+    return (o instanceof Ci.nsIDOMHTMLObjectElement || o instanceof Ci.nsIDOMHTMLEmbedElement)
       && !o.contentDocument && ns.isWindowlessObject(o);
   },
   
@@ -178,17 +222,16 @@ ClearClickHandler.prototype = {
     return doc.__clearClickCanvas || (doc.__clearClickCanvas = doc.createElementNS(HTML_NS, "canvas"));
   },
   
-  isSupported: function(doc) {
-    return true; 
-  },
-  
-  _semanticContainers: [CI.nsIDOMHTMLParagraphElement, CI.nsIDOMHTMLQuoteElement,
-                        CI.nsIDOMHTMLUListElement, CI.nsIDOMHTMLOListElement, CI.nsIDOMHTMLDirectoryElement,
-                        CI.nsIDOMHTMLPreElement, CI.nsIDOMHTMLTableElement ]
+  _semanticContainers: [Ci.nsIDOMHTMLParagraphElement, Ci.nsIDOMHTMLQuoteElement,
+                        Ci.nsIDOMHTMLUListElement, Ci.nsIDOMHTMLOListElement, Ci.nsIDOMHTMLDirectoryElement,
+                        Ci.nsIDOMHTMLPreElement, Ci.nsIDOMHTMLTableElement ]
   ,
   isSemanticContainer: function(o) {
-    for each (var t in this._semanticContainers)
-     if (o instanceof t) return true;
+    let scc = this._semanticContainers;
+    for (let j = scc.length; j-- > 0;)
+     if (o instanceof scc[j]) 
+       return o.ownerDocument.URL.indexOf("view-source") !== 0;
+  
     return false;
   },
   
@@ -196,39 +239,70 @@ ClearClickHandler.prototype = {
     return o.tagName + "/" + (o.tabIndex || 0);
   },
   
-  handle: function(ev) {
+  handleEvent: function(ev) {
+    
     const o = ev.target;
     const d = o.ownerDocument;
-    if (!(d && this.isSupported(d))) return;
+    if (!d) return;
+    
+    if (d === ev.currentTarget || // chrome source, see rapid fire installation
+        ev.button || // right or middle click
+        ev.keyCode && // special keys, e.g. for UI navigation
+        (ev.ctrlKey || ev.metaKey || ev.altKey || ev.keyCode < 48 && ev.keyCode !== 13 && ev.keyCode !== 32)
+      ) {
+      this.rapidFire.ts = 0;
+      // this.ns.log("Reset global event tracking");
+      if (d === ev.currentTarget || ev.keyCode) return;
+    }
+    
     
     const w = d.defaultView;
     if (!w) return;
     
-    const top = w.top;
     const ns = this.ns;
     
-    var isEmbed;
+    const top = w.top;
+    const topURI = top.document.documentURIObject;
+    const topSite = topURI.prePath;
     
-    if (top.__clearClickUnlocked || (top.__clearClickUnlocked = !this.appliesHere(top.location.href)) ||
-        o.__clearClickUnlocked ||
-        o != ev.originalTarget ||
-        o == d.documentElement || o == d.body || // key event on empty region
+    var ts = Date.now();
+    
+    if (this.rapidFire.check(ev, topSite, ts)) {
+      this.swallowEvent(ev);
+      ns.log("[NoScript ClearClick] Swallowed event " + ev.type + " on " + topURI.spec + " (rapid fire from " + this.rapidFire.topSite + " in "  + (ts - this.rapidFire.ts) + "ms)", true);
+      return;
+    }
+    
+    if (!("__clearClickUnlocked" in top)) 
+      top.__clearClickUnlocked = !this.appliesHere(topURI.spec);
+    
+    if (top.__clearClickUnlocked) return;
+    
+    if (!("__clearClickUnlocked" in o)) {
+
+      o.__clearClickUnlocked = 
+        o === d.documentElement || o === d.body || // key event on empty region
         this.isSemanticContainer(o) ||
-        (o.__clearClickUnlocked = !(isEmbed = this.isEmbed(o)) && // plugin embedding?
-            (w == w.top || w.__clearClickUnlocked ||
-              (w.__clearClickUnlocked = this.isWhitelisted(w))
-              || this.sameSiteParents(w)) || // cross-site document?
-        ns.getPluginExtras(o) || // NS placeholder?
-        this.checkSubexception(isEmbed && (o.src || o.data) || w.location.href)
-        )
-      ) return;
+        !(isEmbed = this.isEmbed(o)) && // plugin embedding?
+          (w == top || 
+            ("__clearClickUnlocked" in w 
+              ? w.__clearClickUnlocked
+              : (w.__clearClickUnlocked = this.isWhitelisted(w))
+            ) ||
+            this.sameSiteParents(w) // cross-site document?
+          ) || 
+          ns.getPluginExtras(o) || // NS placeholder?
+           this.checkSubexception(isEmbed && (o.src || o.data) || w.location.href)
+    }
     
+    if (o.__clearClickUnlocked || w.__clearClickUnlocked) return;
+
     var p = ns.getExpando(o, "clearClickProps", {});
     var verbose = ns.consoleDump & LOG_CLEARCLICK;
     var etype = ev.type;
     if (verbose) ns.dump(o.tagName + ", " + etype + ", " + p.toSource());
     
-    var ts = 0, obstructed, ctx, primaryEvent;
+    var obstructed, ctx, primaryEvent;
     try {
       if (etype == "blur") {
         if(/click|key/.test(p.lastEtype)) {
@@ -239,11 +313,10 @@ ClearClickHandler.prototype = {
       }
       if (p.unlocked) return;
     
-      ts = Date.now();
       ctx = /mouse/.test(etype)
                 && { x: ev.pageX, y: ev.pageY, debug: ev.ctrlKey && ev.button == 1 && ns.getPref("clearClick.debug") }
                 || {};
-      ctx.isEmbed = isEmbed;
+      ctx.isEmbed =  typeof(isEmbed) == "boolean" ? isEmbed : this.isEmbed(o);
       
       primaryEvent = /^(?:mousedown|keydown)$/.test(etype) ||
           // submit button generates a syntethic click if any text-control receives [Enter]: we must consider this "primary"
@@ -256,7 +329,7 @@ ClearClickHandler.prototype = {
         ? p.obstructed = this.checkObstruction(o, ctx)
         : p.obstructed; // cache for non-primary events       
     } catch(e) {
-      ns.dump(e.message + ": " + e.stack);
+      ns.dump(e);
       obstructed = true;
     } finally {
       p.lastEtype = etype;
@@ -268,7 +341,7 @@ ClearClickHandler.prototype = {
     
     if (verbose) ns.dump("ClearClick: " + ev.target.tagName + " " + etype +
        "(s:{" + ev.screenX + "," + ev.screenY + "}, p:{" + ev.pageX + "," + ev.pageY + "}, c:{" + ev.clientX + "," + ev.clientY + 
-       ", w:" + ev.which + "}) - obstructed: " + obstructed + ", check time: " + (new Date() - ts) + ", quarantine: " + quarantine +
+       ", w:" + ev.which + "}) - obstructed: " + obstructed + ", check time: " + (Date.now() - ts) + ", quarantine: " + quarantine +
        ", primary: " + primaryEvent + ", ccp:" + (top.__clearClickProps && top.__clearClickProps.toSource()));
     
     var unlocked = !obstructed && primaryEvent && quarantine > 3000;
@@ -281,16 +354,16 @@ ClearClickHandler.prototype = {
       this.swallowEvent(ev);
       ns.log("[NoScript ClearClick] Swallowed event " + etype + " on " + this.forLog(o) + " at " + w.location.href, true);
       var docShell = DOM.getDocShellForWindow(w);
-      var loading = docShell && (docShell instanceof CI.nsIWebProgress) && docShell.isLoadingDocument;
+      var loading = docShell && (docShell instanceof Ci.nsIWebProgress) && docShell.isLoadingDocument;
       if (!loading) {
         p.ts = ts;
         if (primaryEvent && ctx.img && ns.getPref("clearClick.prompt") && !this.prompting) {
           try {
             this.prompting = true;
             var params = {
-              url: isEmbed && (o.src || o.data) || o.ownerDocument.URL,
+              url: ctx.isEmbed && (o.src || o.data) || o.ownerDocument.URL,
               pageURL: w.location.href,
-              topURL: w.top.location.href,
+              topURL: topURI.spec,
               img: ctx.img,
               locked: false,
               pageX: ev.pageX,
@@ -315,7 +388,7 @@ ClearClickHandler.prototype = {
   },
   
   findParentForm: function(o) {
-    var ftype = CI.nsIDOMHTMLFormElement;
+    var ftype = Ci.nsIDOMHTMLFormElement;
     while((o = o.parentNode)) {
       if (o instanceof ftype) return o;
     }
@@ -334,24 +407,23 @@ ClearClickHandler.prototype = {
   minWidth: 160,
   minHeight: 100,
   _NO_SCROLLBARS: {w: 0, h: 0},
-  computeScrollbarSizes: function(frame, dElem, body) {
-    var fw = frame.clientWidth, fh = frame.clientHeight;
-    var dw = dElem.clientWidth, dh = dElem.clientHeight;
-
-    var w = Math.min(fw, dw), h = Math.min(fh, dh);
-    if (body) {
-      var bw = body.clientWidth;
-      var bh = body.clientHeight;
-      if (bw <= fw && (bw - dw > 24 || dw > fw)) w = bw; 
-     
-      if (bh <= fh && (bh - dh > 24 || dh > fh)) h = bh;
-
+  computeScrollbarSizes: function(window, dElem, body) {
+    var fw = window.innerWidth, fh = window.innerHeight;
+    
+    if (body && body.ownerDocument.compatMode == "BackCompat") {
+      dElem = body;
     }
     
-    return { w: fw - w, h: fh - h };
+    var dw = dElem.clientWidth, dh = dElem.clientHeight;
+    var w = Math.min(fw, dw), h = Math.min(fh, dh);
+    
+    var zoom = window.QueryInterface(Ci.nsIInterfaceRequestor)
+            .getInterface(Ci.nsIDOMWindowUtils).screenPixelsPerCSSPixel;
+
+    return { w: (fw - w) * zoom, h: (fh - h) * zoom };
   },
   
-  checkObstruction: function(o, ctx) {   
+  checkObstruction: function(o, ctx) {
     var d = o.ownerDocument;
     var dElem = d.documentElement;
     
@@ -368,7 +440,7 @@ ClearClickHandler.prototype = {
     var bgStyle;
     var box, curtain;
     
-    var frame, frameClass, frameStyle, objClass, viewer;
+    var frame, frameClass, objClass;
     
     var docPatcher = new DocPatcher(this.ns, o, w);
     
@@ -400,6 +472,7 @@ ClearClickHandler.prototype = {
        
       docPatcher.linkAlertHack(true);
       docPatcher.fbPresenceHack(true);
+      docPatcher.abpTabsHack(true);
       
       try {
         docPatcher.opaque(true);
@@ -408,8 +481,6 @@ ClearClickHandler.prototype = {
         
         if (ctx.isEmbed) { // objects and embeds
           if (this.ns.getPref("clearClick.plugins", true)) {
-            var ds = browser.docShell;
-            viewer = ds.contentViewer && false;
             objClass = new ClassyObj(o);
             objClass.append(" __noscriptBlank__");
             docPatcher.blankPositioned(true);
@@ -422,15 +493,12 @@ ClearClickHandler.prototype = {
         if ((frame = w.frameElement)) {
           frameClass = new ClassyObj(frame);
           DOM.removeClass(frame, "__noscriptScrolling__");
-          sd = this.computeScrollbarSizes(frame, dElem, d.body);
-          var zoom = d.defaultView.QueryInterface(CI.nsIInterfaceRequestor)
-            .getInterface(CI.nsIDOMWindowUtils).screenPixelsPerCSSPixel;
-          sd.w *= zoom;
-          sd.h *= zoom;
+          sd = this.computeScrollbarSizes(w, dElem, d.body);
+          
         }
         
-        var clientHeight = w.innerHeight - sd.h;
-        var clientWidth =  w.innerWidth - sd.w;
+        let clientHeight = w.innerHeight - sd.h;
+        let clientWidth =  w.innerWidth - sd.w;
         // print(dElem.clientWidth + "," +  dElem.clientHeight + " - "  + w.innerWidth + "," + w.innerHeight);
         
         if (!ctx.isEmbed) {
@@ -447,24 +515,18 @@ ClearClickHandler.prototype = {
             
             background = this.rndColor();
           }
-          frameStyle = w.parent.getComputedStyle(frame, '');
         }     
         
-        if (curtain && frame) {
-          dElem.appendChild(curtain);
-        }
-        
-        var maxWidth = Math.max(Math.min(this.maxWidth, clientWidth), this.minWidth);
-        var maxHeight = Math.max(Math.min(this.maxHeight, clientHeight), this.minHeight);
+        let maxWidth = Math.max(Math.min(this.maxWidth, clientWidth), sd.w ? 0 : Math.min(this.minWidth, dElem.offsetWidth), 8);
+        let maxHeight = Math.max(Math.min(this.maxHeight, clientHeight), sd.h ? 0 : Math.min(this.minHeight, dElem.offsetHeight, 8));
   
         box = this.getBox(o, d, w);
         
         // expand to parent form if needed
         var form = o.form;
-        var formBox = null;
         if (frame && !ctx.isEmbed && (form || (form = this.findParentForm(o)))) {
   
-          formBox = this.getBox(form, d, w);
+          let formBox = this.getBox(form, d, w);
           if (!(formBox.width && formBox.height)) { // some idiots put <form> as first child of <table> :(
             formBox = this.getBox(form.offsetParent || form.parentNode, d, w);
             if (!(formBox.width && formBox.height)) {
@@ -473,10 +535,15 @@ ClearClickHandler.prototype = {
           }
     
           if (formBox.width && formBox.height) {
-            ctx.x = ctx.x || box.x + box.width;
-            ctx.y = ctx.y || box.y + box.height;
-            box = formBox;
+            // form has layout, recenter to show as much as possible
+            ctx.x = ctx.x || box.x + box.width;   // use mouse coordinates or
+            ctx.y = ctx.y || box.y + box.height; // rightmost widget position 
+            
+            box = formBox; // the form is our new reference
+            
             var delta;
+            
+            // move inside the viewport if needed
             if (box.x < 0) {
               box.screenX -= box.x;
               box.x = 0;
@@ -485,7 +552,9 @@ ClearClickHandler.prototype = {
               box.screenY -= box.y;
               box.y = 0;
             }
-            if (box.x + Math.min(box.width, maxWidth) < ctx.x) {
+            
+            // is our center out of the form?
+            if (box.x + Math.min(box.width, maxWidth) < ctx.x) { 
               box.width = Math.min(box.width, maxWidth);
               delta = ctx.x + 4 - box.width - box.x;
               box.x += delta;
@@ -498,6 +567,11 @@ ClearClickHandler.prototype = {
               box.y += delta;
               box.screenY += delta;
             }
+            
+            // recenter to the form
+            ctx.x = box.x + box.width / 2;
+            ctx.y = box.y + box.height / 2;
+            
             o = form;
           }
         }
@@ -511,10 +585,12 @@ ClearClickHandler.prototype = {
           x: w.scrollX, 
           y: w.scrollY, 
           width: Math.max(w.innerWidth - sd.w, 32), 
-          height: Math.max(w.innerHeight - sd.h, 32),
+          height: Math.max(w.innerHeight - sd.h, 16), // Facebook like buttons are 20 pixel high
           frame: frame
         };
-
+        
+        var rtlOffset = 0;
+        
         if (ctx.isEmbed) { // check in-page vieport
           vp.frame = null;
           vp.x = Math.max(vp.x, box.x);
@@ -522,40 +598,73 @@ ClearClickHandler.prototype = {
           vp.width = Math.min(vp.width, box.width);
           vp.height = Math.min(vp.height, box.height);
           
-          for(form = o; form = form.parentNode;) {
+          for(let ancestor = o; ancestor = ancestor.parentNode;) {
   
-            if ((form.offsetWidth < box.width || form.offsetHeight < box.height) &&
-                w.getComputedStyle(form, '').overflow != "visible") {
+            if ((ancestor.offsetWidth < box.width || ancestor.offsetHeight < box.height) &&
+                w.getComputedStyle(ancestor, '').overflow != "visible") {
               
               // check if we're being fooled by some super-zoomed applet
-              if (box.width / 4 <= form.offsetWidth && box.height / 4 <= form.offsetHeight) {
-                formBox = this.getBox(form, d, w);
+              if (box.width / 4 <= ancestor.offsetWidth && box.height / 4 <= ancestor.offsetHeight) {
+                let ancestorBox = this.getBox(ancestor, d, w);
                 
-                if (box.x < formBox.x) {
-                  box.x = formBox.x;
-                  box.screenX = formBox.screenX;
+                if (box.x < ancestorBox.x) {
+                  box.x = ancestorBox.x;
+                  box.screenX = ancestorBox.screenX;
                 }
-                if (box.y < formBox.y) { 
-                  box.y = formBox.y;
-                  box.screenY = formBox.screenY;
+                if (box.y < ancestorBox.y) { 
+                  box.y = ancestorBox.y;
+                  box.screenY = ancestorBox.screenY;
                 }
-                if (box.width + box.x > formBox.width + formBox.x) box.width = Math.max(this.minWidth, form.clientWidth - (box.x - formBox.x));
-                if (box.height + box.y > formBox.height + formBox.y) box.height = Math.max(this.minHeight, form.offsetHeight - (box.y - formBox.y));
+                if (box.width + box.x > ancestorBox.width + ancestorBox.x) box.width = Math.max(this.minWidth, ancestor.clientWidth - (box.x - ancestorBox.x));
+                if (box.height + box.y > ancestorBox.height + ancestorBox.y) box.height = Math.max(this.minHeight, ancestor.offsetHeight - (box.y - ancestorBox.y));
               }
               break;
             }
           }
-        } else if (!(sd.w || sd.h)) { // no scrollbars
-          if (!sd.w) {
-            vp.x = 0;
-            vp.width = curtain.offsetWidth;
-          }
-          if (!sd.h) {
-            vp.y = 0;
-            vp.height = curtain.offsetHeight;
+        } else {
+          
+          // correct x offsets according to left scrollbars if needed
+          try {
+            var adaptiveScrollerSide = false;
+            switch(this.ns.prefService.getIntPref("layout.scrollbar.side")) {  
+              case 1:
+                adaptiveScrollerSide = true;
+              case 0:
+                if (!adaptiveScrollerSide && this.ns.prefService.getIntPref("bidi.direction") != 2) 
+                  break;
+              case 3:
+                vp.x += this._scrollerCorrect(w, adaptiveScrollerSide);
+                rtlOffset = this._scrollerCorrect(top, adaptiveScrollerSide);
+            }
+          } catch(e) {
+            if (ns.consoleDump & LOG_CLEARCLICK) ns.dump(e);
           }
+          
+         
+          
         }
         
+        // clip viewport intersecting with scrolling parents
+        
+        const CLIP_MIN = 64;
+        var clip = this._clip(o.parentNode, frame ? this.getBox(frame) : box);
+        if (clip.h != 0) {
+          if (vp.height + clip.h >= CLIP_MIN) vp.height += clip.h;
+          else vp.height = CLIP_MIN;
+          if (maxHeight + clip.h >= CLIP_MIN) maxHeight += clip.h;
+          else maxHeight = CLIP_MIN;
+        }
+        if (clip.w != 0) {
+          if (vp.width + clip.w >= CLIP_MIN) vp.width += clip.w;
+            else vp.width = CLIP_MIN;
+            if (maxWidth + clip.w >= CLIP_MIN) maxWidth += clip.w;
+            else maxWidth = CLIP_MIN;
+        }
+        vp.x += clip.x;
+        vp.y += clip.y;
+        
+        // Fit in viewport
+        
         box.oX = box.x;
         box.oY = box.y;
         box.oW = box.width;
@@ -567,14 +676,18 @@ ClearClickHandler.prototype = {
         this._constrain(box, "y", "height", maxHeight, vp, ctx.y);
         // print(box.toSource());     
         
+        
+         
         c.width = box.width;
         c.height = box.height;
-        
+         
         
         if (this.ns.consoleDump & LOG_CLEARCLICK) this.ns.dump("Snapshot at " + box.toSource() + " + " + w.pageXOffset + ", " + w.pageYOffset);
-        
-        
-        
+          
+        if (curtain && frame) {
+          dElem.appendChild(curtain);
+        }
+
         img1 = snapshot(w, box.x, box.y);
       
       } finally {
@@ -585,8 +698,10 @@ ClearClickHandler.prototype = {
       var rootElement = top.document.documentElement;
       var rootBox = this.getBox(rootElement, top.document, top);
       
-      var offsetX = (box.screenX - rootBox.screenX);
+      
       var offsetY = (box.screenY - rootBox.screenY);
+      var offsetX = (box.screenX - rootBox.screenX) + rtlOffset;
+      
       var ret = true;
       var tmpImg;
       
@@ -686,18 +801,86 @@ ClearClickHandler.prototype = {
       if (typeof(bgStyle) == "string") dElem.style.background = bgStyle;
      
       docPatcher.opaque(false);
-      docPatcher.linkAlertHack(false);
+      docPatcher.abpTabsHack(false);
       docPatcher.fbPresenceHack(false);
+      docPatcher.linkAlertHack(false);
       
       if (objClass) objClass.reset();
       if (frameClass) frameClass.reset();
-      if (viewer) viewer.enableRendering = true;
     }
     
     return ret;
  
-  }
+  },
   
+  _clip: function(parent, box) {
+    const MIN = 64;
+    
+    // backtrack all the overflow~="auto|scroll" parent elements and clip
+        
+    var pw = parent.ownerDocument.defaultView;
+
+    var current, cbox;
+    var dw = 0, dh = 0, dx = 0, dy = 0;
+
+    var bx = box.screenX;
+    var by = box.screenY;
+    var bw = box.width;
+    var bh = box.height;
+    
+    const ELEMENT = Ci.nsIDOMElement;
+    
+    while(parent) {
+   
+      current = parent; 
+      switch (pw.getComputedStyle(current, '').overflow) {
+        case "auto" : case "scroll":
+        cbox = this.getBox(current);
+        
+        d = cbox.screenY - by;
+        if (d > 0) {
+          dy += d;
+          dh -= d;
+          by += d;
+          bh -= d;
+        }
+        d = cbox.screenX - bx;
+        if (d > 0) {
+          dx += d;
+          dw -= d;
+          bx += d;
+          bw -= d;
+          
+        }
+        d = by + bh - (cbox.screenY + current.clientHeight);
+        if (d > 0) {
+          if (cbox.height - current.clientHeight < 10) // compensate for miscalculated scrollbars bug
+            d += 20;
+          dh -= d;
+          bh -= d;
+        }
+        d = bx + bw - (cbox.screenX + current.clientWidth);
+        if (d > 0) {
+          if (cbox.width - current.clientWidth < 10) // compensate for miscalculated scrollbars bug
+            d += 20;
+          dw -= d;
+          bw -= d;
+        }
+      }
+      parent = current.parentNode;
+      if (parent instanceof ELEMENT) continue;
+      parent = pw.frameElement;
+      if (parent) pw = parent.ownerDocument.defaultView;
+    }
+    
+    return { x: dx, y: dy, w: dw, h: dh };
+  },
+  
+  _scrollerCorrect: function(w, adaptive) {
+    return (adaptive && w.getComputedStyle(w.document.body || w.document.documentElement, '').direction != 'rtl')
+      ? 0
+      : w.innerWidth - w.document.documentElement.clientWidth;
+  }
 }
 
 function ClassyObj(o) {
@@ -765,7 +948,7 @@ DocPatcher.prototype = {
     var hasPos = false;
     const posn = [];
     
-    const tw = d.createTreeWalker(d, CI.nsIDOMNodeFilter.SHOW_ELEMENT, null, false);
+    const tw = d.createTreeWalker(d, Ci.nsIDOMNodeFilter.SHOW_ELEMENT, null, false);
     for (var n = null; (n = tw.nextNode());) {
       b = this.getRect(n, d);
       if (b.bottom < top || b.top > bottom ||
@@ -873,32 +1056,13 @@ DocPatcher.prototype = {
     }
   },
   
-  get oldStyle() {
-    delete this.__proto__.oldStyle;
-    return this.__proto__.oldStyle = Components.ID('{41d979dc-ea03-4235-86ff-1e3c090c5630}')
-                 .equals(CI.nsIStyleSheetService);
-  },
-  applySheet: function(sheet) {
-    var sheetHandle = null;
-    if (this.oldStyle) {
-      // Gecko < 1.9, asynchronous user sheets force ugly work-around
-      var d = this.o.ownerDocument;
-      sheetHandle = d.createElementNS(HTML_NS, "style");
-      sheetHandle.innerHTML = sheet;
-      d.documentElement.appendChild(sheetHandle);
-    } else { // 
-      this.ns.updateStyleSheet(sheetHandle = sheet, true);
-    }
+
+  applySheet: function(sheetHandle) {
+    this.ns.updateStyleSheet(sheetHandle, true);
     return sheetHandle;
   },
   removeSheet: function(sheetHandle) {
-    if (sheetHandle) {
-      if (this.oldStyle) {
-        this.o.ownerDocument.documentElement.removeChild(sheetHandle);
-      } else {
-        this.ns.updateStyleSheet(sheetHandle, false);
-      }
-    }
+    this.ns.updateStyleSheet(sheetHandle, false);
   },
   
   _linkAlertBox: null,
@@ -931,17 +1095,85 @@ DocPatcher.prototype = {
   _fbPresence: null,
   fbPresenceHack: function(toggle) {
     if (toggle) {
-      if (this.top.location.host == "apps.facebook.com") {
-        var fbPresence = this.top.document.getElementById("presence");
-        if (fbPresence) {
-          fbPresence._ccVisibility = fbPresence.style.visibility;
-          fbPresence.style.visibility = "hidden";
-          this._fbPresence = fbPresence; 
+      try {
+        if (this.top.location.host == "apps.facebook.com") {
+          var fbPresence = this.top.document.getElementById("presence");
+          if (fbPresence) {
+            fbPresence._ccVisibility = fbPresence.style.visibility;
+            fbPresence.style.visibility = "hidden";
+            this._fbPresence = fbPresence; 
+          }
         }
-      }
+      } catch(e) {}
     } else if (this._fbPresence) {
       this._fbPresence.style.visibility = this._fbPresence._ccVisibility;
     }
-  }
+  },
   
-}
\ No newline at end of file
+  _abpTabs: null,
+  get _abpTabsObj() {
+    delete this.__proto__._abpTabsObj;
+    var tobj;
+    try {
+      tobj = Cc["@mozilla.org/adblockplus;1"].getService().wrappedJSObject.objTabs;
+    } catch(e) {
+      try {
+        let baseURL = Cc["@adblockplus.org/abp/private;1"].getService(Ci.nsIURI);
+        let x = {};
+        Cu.import(baseURL.spec + "ObjectTabs.jsm", x);
+        tobj = Cu.getGlobalForObject(x.objectMouseEventHander).objTabs;
+      } catch (e) {
+        tobj = null;
+      }
+    }
+    return this.__proto__._abpTabsObj = tobj;
+  },
+  
+  abpTabsHack: function(toggle) {
+    let visibleClass, hiddenClass;
+    try {
+      let tobj = this._abpTabsObj;
+      if (tobj) {
+        hiddenClass = tobj.objTabClassHidden;
+        if (!hiddenClass) return;
+        visibleClasses = [tobj.objTabClassVisibleTop, tobj.objTabClassVisibleBottom];
+      }
+    } catch(e) {
+    }
+    
+    (this.__proto__.abpTabsHack =
+      hiddenClass
+      ? function(toggle) {
+          try {
+            let document = this.top.document;
+            if (toggle) {
+              let tabs = [];
+              for each(let c in visibleClasses) {
+                
+                Array.forEach(document.getElementsByClassName(c), function(t) {
+                  let co = new ClassyObj(t);
+                  t.className = hiddenClass;
+                  tabs.push(co);
+                });
+              }
+              this._abpTabs = tabs;
+            } else {
+              for (let co in this._abpTabs) {
+                co.reset();
+              }
+            }
+          } catch(e) {}
+        }
+      : DUMMY_FUNC
+     )(toggle); 
+  }
+};
+
+
+["clearClick", "clearClick.exceptions", "clearClick.subexceptions"].forEach(function(p) {
+  try {
+    ns.syncPrefs(ns.prefs, p);
+  } catch(e) {
+    ns.dump(e.message + ":" + e.stack + " setting " + p + "\n");
+  }
+});
\ No newline at end of file
