@@ -1,9 +1,3 @@
-const CP_OK = 1;
-const CP_NOP = function() { return CP_OK };
-const CP_FRAMECHECK = 2;
-const CP_SHOULDPROCESS = 4;
-const CP_EXTERNAL = 0;
-
 const PolicyState = {
   _debug: false,
   _uris: [],
@@ -25,7 +19,7 @@ const PolicyState = {
   
   attach: function(channel) {
     if (this.URI === channel.URI) {
-      if (this._debug) this.push(this.URI);
+      if (this._debug) this._uris.push(this.URI);
       IOUtil.attachToChannel(channel, "noscript.policyHints", this.hints);
       this.reset();
     }
@@ -33,7 +27,7 @@ const PolicyState = {
   extract: function(channel, detach) {
     var res = IOUtil.extractFromChannel(channel, "noscript.policyHints", !detach);
     if (detach && res !== null && this._debug) {
-      var idx = this._uris.indexOf(this.URI);
+      var idx = this._uris.indexOf(channel.URI);
       if (idx > -1) this._uris.splice(idx, 1);
     }
     return res;
@@ -61,18 +55,21 @@ const PolicyState = {
 }
 
 function PolicyHints(hints) {
-  Array.prototype.push.apply(this, Array.slice(hints, 0));
+  this.push.apply(this, Array.slice(hints, 0));
+  this.wrappedJSObject = this;
 }
 
 PolicyHints.prototype = (function() {
-  var proto = new Array();
+  const proto = {
+    __proto__: [],
+    toArray: function() Array.slice(this, 0),
+    toSource: Object.prototype.toSource,
+    toString:  Object.prototype.toSource
+  };
   ["contentType", "contentLocation", "requestOrigin", "context", "mimeType", "extra"].forEach(function(p, i) {
-    this.__defineGetter__(p, function() { return this[i] });
-    this.__defineSetter__(p, function(v) { return this[i] = v });
+    this.__defineGetter__(p, function() this[i]);
+    this.__defineSetter__(p, function(v) this[i] = v);
    }, proto);
-   proto.toArray = function() {
-      return Array.slice(this, 0)
-   };
    return proto;
 })();
 
@@ -110,46 +107,52 @@ const MainContentPolicy = {
           forbid, isScript, isJava, isFlash, isSilverlight,
           isLegacyFrame, blockThisFrame, contentDocument,
           logIntercept, logBlock,
-          unwrappedLocation, mimeKey;
+          unwrappedLocation, mimeKey,
+          mustCountObject = false;
       
       logIntercept = this.consoleDump;
       if(logIntercept) {
         logBlock = logIntercept & LOG_CONTENT_BLOCK;
         logIntercept = logIntercept & LOG_CONTENT_INTERCEPT;
       } else logBlock = false;
-    
-      if (aContentType == 1 && !this.POLICY_OBJSUB) { // compatibility for type OTHER
-        if (aContext instanceof CI.nsIDOMHTMLDocument) {
-          aContentType = arguments.callee.caller ? 11 : 9;
-        } else if ((aContext instanceof CI.nsIDOMHTMLElement)) {
-          if ((aContext instanceof CI.nsIDOMHTMLEmbedElement || aContext instanceof CI.nsIDOMHTMLObjectElement)) {
-            aContentType = 12;
-          } else if (aContext.getAttribute("ping")) {
-            aContentType = 10;
-          }
-        }
-        arguments[0] = aContentType;
-      }
-      
+
       unwrappedLocation = IOUtil.unwrapURL(aContentLocation);
       scheme = unwrappedLocation.scheme;
       
-      var isHTTP = /^https?$/.test(scheme);
+      var isHTTP = scheme === "http" || scheme === "https";
       
       if (isHTTP) {
-        
-        HTTPS.forceURI(unwrappedLocation, null, aContext);
-        
-        if (aRequestOrigin && !aInternalCall) {
-          
-          if (aContentType != 3) // images are a bitch if cached!
-            XOriginCache.store(aRequestOrigin, arguments.xOriginKey = unwrappedLocation);
+        if (aRequestOrigin) {
           
           switch(aContentType) {
-            // case 2: case 4: // scripts stall if blocked later
+            case 5:
+              // early ABE check for any plugin content except Flash, Silverlight and PDF
+              // (Java, for instance, is known to bypass HTTP observers!)
+              if (/^application\/(?:x-(?:shockwave-flash|silverlight)$|futuresplash|pdf$)/i.test(aMimeTypeGuess))
+                break;
+            
+            case 7:
+            
+              if ((aContext instanceof Ci.nsIDOMHTMLObjectElement) &&
+                  this.isCachedObjectMime(aMimeTypeGuess) &&
+                  !(aContext.offsetWidth && aContext.offsetHeight) &&
+                  this.getPref("allowCachingObjects") &&
+                  aRequestOrigin && this.isJSEnabled(this.getSite(aRequestOrigin.spec)) &&
+                  !this.pluginForMime(aMimeTypeGuess) &&
+                  (aMimeTypeGuess.indexOf("css") > 0 || this.isJSEnabled(this.getSite(aContentLocation.spec)))
+                 ) {
+                return CP_OK;
+              }
+              
+              if (aContentType === 7 || aInternalCall) break;
+              
             case 1: case 12: // we may have no chance to check later for unknown and sub-plugin requests
-              if (ABE.checkPolicy(aRequestOrigin, unwrappedLocation))
+              HTTPS.forceURI(unwrappedLocation, null, aContext);
+              let res = ABE.checkPolicy(aRequestOrigin, unwrappedLocation, aContentType);
+              if (res && res.fatal) {
+                this.requestWatchdog.notifyABE(res, true);
                 return this.reject("ABE-denied inclusion", arguments); 
+              }
           }
         }
         
@@ -181,31 +184,24 @@ const MainContentPolicy = {
           return this.reject("Ping", arguments);
             
         case 2:
-          if (this.forbidChromeScripts && this.checkForbiddenChrome(aContentLocation, aRequestOrigin)) {
-            return this.reject("Chrome Access", arguments);
-          }
-          if (this.forbidJarDocuments && aRequestOrigin && this.checkJarDocument(aContentLocation, aContext, aRequestOrigin)) {
-            return this.reject("Cross-site jar-embedded script", arguments);
-          }
           forbid = isScript = true;
           break;
         case 3: // IMAGES
           if (this.blockNSWB && 
               !(this.jsEnabled || aRequestOrigin &&
                 ((originSite = this.getSite(aRequestOrigin.spec)) == this.getSite(aContentLocation.spec)
-                    || this.isJSEnabled(originSite)))
-                && aContext instanceof CI.nsIDOMHTMLImageElement) {
+                    || this.isJSEnabled(originSite) && !(this.getExpando(aContext.ownerDocument, "nselForce"))))
+                && aContext instanceof Ci.nsIDOMHTMLElement) {
             try {
-              for (var parent = aContext; (parent = parent.parentNode);) {
-                if (parent.nodeName.toUpperCase() == "NOSCRIPT")
+              for (var parent = aContext; (parent = parent.parentNode);)
+                if (parent.__nselForce || parent.nodeName.toUpperCase() == "NOSCRIPT")
                   return this.reject("Tracking Image", arguments);
-              }
             } catch(e) {
               this.dump(e)
             }
           }
   
-          PolicyState.cancel(arguments);
+          // PolicyState.cancel(arguments); // needed for ABE INCLUSION(IMAGE)
           return CP_OK;
         
         case 4: // STYLESHEETS
@@ -215,26 +211,66 @@ const MainContentPolicy = {
             forbid = isScript = true; // we treat XSLT like scripts
             break;
           }
-          
-          if (this.forbidJarDocuments && aRequestOrigin && this.checkJarDocument(aContentLocation, aContext, aRequestOrigin)) {
-            return this.reject("Cross-site jar-embedded stylesheet", arguments);
-          }
-          
           return CP_OK;
         
         case 14: // fonts
           forbid = this.forbidFonts;
           if (!forbid) return CP_OK;
           mimeKey = "Font";
+          if (aContentLocation && aRequestOrigin && aContentLocation.schemeIs("data"))
+            locationURL = this.getSite(aRequestOrigin.spec);
+          
           break;
         
         case 5: // embeds
+          // here we account for sloppy handling of the archive and code attributes on applets
+          if (!aInternalCall) {
+              
+            let archive = aContext.getAttribute("archive");
+            let code = '';
+            if (aContext instanceof Ci.nsIDOMHTMLEmbedElement) {
+              code = aContext.getAttribute("code"); 
+              if (code && /\bjava\b/.test(aMimeTypeGuess)) {
+                archive = archive ? code + " " + archive : code;
+              } else code = '';
+            }
+            if (archive) {
+              let prePaths;
+              let base = aContentLocation;       
+              let jars = archive.split(/[\s,]+/)
+              for (let j = jars.length; j-- > 0;) {
+                try {
+                  let jar = jars[j];
+                  let u = IOUtil.newURI(base.resolve(jar));
+                  let prePath = u.prePath;
+                  if (prePath !== base.prePath) {
+                    if (prePaths) {
+                      if (prePaths.indexOf(prePath) !== -1) continue;
+                      prePaths.push(prePath);
+                    } else prePaths = [prePath];
+                  } else {
+                    if (j === 0 && code === jar) aContentLocation = u;
+                    continue;
+                  }
+                  this.setExpando(aContext, "allowed", null);
+                  let res = this.shouldLoad(aContentType, u, aRequestOrigin, aContext, aMimeTypeGuess, CP_OBJECTARC);
+                  if (res !== CP_OK) return res;
+                } catch (e) {
+                  this.dump(e)
+                }
+              }
+              this.setExpando(aContext, "allowed", null);
+            }
+          }
+  
+          
+          
         case 15: // media
           if (aContentLocation && aRequestOrigin && 
               (locationURL = aContentLocation.spec) == (originURL = aRequestOrigin.spec) &&
               aMimeTypeGuess) {
               
-            if ((aContext instanceof CI.nsIDOMHTMLEmbedElement) &&
+            if ((aContext instanceof Ci.nsIDOMHTMLEmbedElement) &&
               this.isAllowedObject(locationURL, aMimeTypeGuess)
               ) {
               if (logIntercept) this.dump("Plugin document " + locationURL);
@@ -251,10 +287,6 @@ const MainContentPolicy = {
             }
           }
           
-          if (this.checkJarDocument(aContentLocation, aContext)) 
-            return this.reject("Plugin content from JAR", arguments);
-          
-          
           if (aContentType == 15) {
               if (aRequestOrigin && !this.isJSEnabled(this.getSite(aRequestOrigin.spec))) {
               // let's wire poor man's video/audio toggles if JS is disabled and therefore controls are not available
@@ -273,14 +305,17 @@ const MainContentPolicy = {
           
           if (aMimeTypeGuess) // otherwise let's treat it as an iframe
             break;
-          
-         
-          
+                    
         case 7:
+          
           locationURL = aContentLocation.spec;
           originURL = aRequestOrigin && aRequestOrigin.spec;
+          
+          if (locationURL.indexOf("view-source:") === 0 && /^(?:https?|ftp):/.test(originURL))
+            return this.reject("Embedded view-source:", arguments);
+          
           if (locationURL == "about:blank" || /^chrome:/.test(locationURL)
-            || !originURL && (aContext instanceof CI.nsIDOMXULElement)  // custom browser like in Stumbleupon discovery window
+            || !originURL && (aContext instanceof Ci.nsIDOMXULElement)  // custom browser like in Stumbleupon discovery window
           ) return CP_OK;
           
           if (!aMimeTypeGuess) {
@@ -299,58 +334,47 @@ const MainContentPolicy = {
             break; // we just need to guess the Mime for video/audio
           }
           
-          isLegacyFrame = aContext instanceof CI.nsIDOMHTMLFrameElement;
-     
-          if (isLegacyFrame
-              ? this.forbidFrames || // we shouldn't allow framesets nested inside iframes, because they're just as bad
-                                     this.forbidIFrames &&
-                                     (aContext.ownerDocument.defaultView.frameElement instanceof CI.nsIDOMHTMLIFrameElement) &&
-                                     this.getPref("forbidMixedFrames", true)
-              : this.forbidIFrames || // we use iframes to make placeholders for blocked legacy frames...
-                                     this.forbidFrames &&
-                                     this.isLegacyFrameReplacement(aContext)
-             ) {
-              try {
-                contentDocument = aContext.contentDocument;
-              } catch(e) {}
-           
-              blockThisFrame = (aInternalCall === CP_FRAMECHECK) && !(
-                      this.knownFrames.isKnown(locationURL, originSite = this.getSite(originURL)) ||
-                    /^(?:chrome|resource|wyciwyg):/.test(locationURL) ||
-                    locationURL == this._silverlightInstalledHack ||
-                    locationURL == this.compatGNotes ||
-                    (
-                      originURL
-                        ? (/^chrome:/.test(originURL) ||
-                           /^(?:data|javascript):/.test(locationURL) &&
-                            (contentDocument && (originURL == contentDocument.URL
-                                                  || /^(?:data:|javascript:|about:blank$)/.test(contentDocument.URL)
-                            ) || this.isFirebugJSURL(locationURL)
-                           )
-                          )
-                        : contentDocument && 
-                          this.getSite(contentDocument.URL) == (locationSite = this.getSite(locationURL))
-                     )
-                ) && this.forbiddenIFrameContext(originURL || (originURL = aContext.ownerDocument.URL), locationURL);
-          }
-          
+          if (!(aContext instanceof Ci.nsIDOMXULElement)) {
+        
+            isLegacyFrame = aContext instanceof Ci.nsIDOMHTMLFrameElement;
+       
+            if (isLegacyFrame
+                ? this.forbidFrames || // we shouldn't allow framesets nested inside iframes, because they're just as bad
+                                       this.forbidIFrames &&
+                                       (aContext.ownerDocument.defaultView.frameElement instanceof Ci.nsIDOMHTMLIFrameElement) &&
+                                       this.getPref("forbidMixedFrames", true)
+                : this.forbidIFrames || // we use iframes to make placeholders for blocked legacy frames...
+                                       this.forbidFrames &&
+                                       this.isLegacyFrameReplacement(aContext)
+               ) {
+                try {
+                  contentDocument = aContext.contentDocument;
+                } catch(e) {}
+             
+                blockThisFrame = (aInternalCall === CP_FRAMECHECK) && !(
+                        this.knownFrames.isKnown(locationURL, originSite = this.getSite(originURL)) ||
+                      /^(?:chrome|resource|wyciwyg):/.test(locationURL) ||
+                      locationURL == this._silverlightInstalledHack ||
+                      locationURL == this.compatGNotes ||
+                      (
+                        originURL
+                          ? ( /^(?:chrome|about|resource):/.test(originURL) && originURL !== "about:blank" || 
+                             /^(?:data|javascript):/.test(locationURL) &&
+                              (contentDocument && (originURL == contentDocument.URL
+                                                    || /^(?:data:|javascript:|about:blank$)/.test(contentDocument.URL)
+                              ) || this.isFirebugJSURL(locationURL)
+                             )
+                            )
+                          : contentDocument && 
+                            this.getSite(contentDocument.URL) == (locationSite = this.getSite(locationURL))
+                       )
+                  ) && this.forbiddenIFrameContext(originURL || (originURL = aContext.ownerDocument.URL), locationURL);
+            }
+          } 
         case 6:
-  
-          if (this.checkJarDocument(aContentLocation, aContext)) 
-            return this.reject("JAR Document", arguments);
-          
-         
           
           if (aRequestOrigin && aRequestOrigin != aContentLocation) {
             
-            if (this.safeToplevel && (aContext instanceof CI.nsIDOMChromeWindow) &&
-                aContext.isNewToplevel &&
-                !(/^(?:chrome|resource|file)$/.test(scheme) ||
-                  this.isSafeJSURL(aContentLocation.spec))
-                  ) {
-              return this.reject("Top Level Window Loading", arguments);
-            }
-         
             if (isHTTP) {
               
               // external?
@@ -359,18 +383,13 @@ const MainContentPolicy = {
                 this.requestWatchdog.externalLoad = aContentLocation.spec;
               }
               
-            } else if(/^(?:data|javascript)$/.test(scheme)) {
-              //data: and javascript: URLs
-              locationURL = locationURL || aContentLocation.spec;
-              originSite = this.getSite(originURL = originURL || aRequestOrigin.spec);
-              if (this.forbiddenJSDataDoc(locationURL, originSite, aContext)) {
-                return this.reject("JavaScript/Data URL", arguments);
-              }
-            } else if(scheme != aRequestOrigin.scheme && 
-                scheme != "chrome" && // faster path for common case
+            } else if(scheme === "data" || scheme === "javascript") {
+              return CP_OK; // JavaScript execution policies will take care of this
+            } else if(scheme !== aRequestOrigin.scheme && 
+                scheme !== "chrome" && // faster path for common case
                 this.isExternalScheme(scheme)) {
               // work-around for bugs 389106 & 389580, escape external protocols
-              if (aContentType != 6 && !aInternalCall && 
+              if (aContentType !== 6 && !aInternalCall && 
                   this.getPref("forbidExtProtSubdocs", true) && 
                   !this.isJSEnabled(originSite = this.getSite(originURL = originURL || aRequestOrigin.spec)) &&
                   (!aContext.contentDocument || aContext.contentDocument.URL != originURL)
@@ -380,7 +399,7 @@ const MainContentPolicy = {
               if (!this.normalizeExternalURI(aContentLocation)) {
                 return this.reject("Invalid External URL", arguments);
               }
-            } else if(aContentType == 6 && scheme == "chrome" &&
+            } else if(aContentType === 6 && scheme === "chrome" &&
               this.getPref("lockPrivilegedUI", false) && // block DOMI && Error Console
               /^(?:javascript:|chrome:\/\/(?:global\/content\/console|inspector\/content\/inspector|venkman\/content\/venkman)\.xul)$/
                 .test(locationURL)) {
@@ -390,6 +409,7 @@ const MainContentPolicy = {
           
           if (!(this.forbidSomeContent || this.alwaysBlockUntrustedContent) ||
                 !blockThisFrame && (
+                  aContext instanceof Ci.nsIDOMXULElement ||
                   !aMimeTypeGuess 
                   || aMimeTypeGuess.substring(0, 5) == "text/"
                   || aMimeTypeGuess == "application/xml" 
@@ -399,7 +419,7 @@ const MainContentPolicy = {
                 )
             ) {
             
-            if (aContext instanceof CI.nsIDOMElement) {
+            if (aContext instanceof Ci.nsIDOMElement) {
               // this is alternate to what we do in countObject, since we can't get there
               // this.delayExec(this.opaqueIfNeeded, 0, aContext); // TODO uncomment
             }
@@ -408,8 +428,8 @@ const MainContentPolicy = {
               this.dump("Document OK: " + aMimeTypeGuess + "@" + (locationURL || aContentLocation.spec) + 
                 " --- PGFM: " + this.pluginForMime(aMimeTypeGuess));
             
-            if (aContentLocation.schemeIs("about") && /^about:(net|cert)error?/.test(aContentLocation.spec)) {
-              ns.handleErrorPage(aContext, aContentLocation);
+            if (aContentLocation.schemeIs("about") && /^about:(?:net|cert)error\?/.test(aContentLocation.spec)) {
+              this.handleErrorPage(aContext, aContentLocation);
             }
             
             return CP_OK;
@@ -445,35 +465,60 @@ const MainContentPolicy = {
         
         contentDocument = aContext && aContext.ownerDocument || aContext;
         
-        if (aContentType == 2) { // "real" JavaScript include
-        
-          // plugin instantiation hacks
-          if (this.contentBlocker && originSite && /^(?:https?|file):/.test(originSite)) {
-            this.applyPluginPatches(contentDocument);
-          }
-                  
-          forbid = !(originSite && locationSite == originSite);
-        } else isScript = false;
-        
         if (contentDocument) // XSLT comes with no context sometimes...
           this.getExpando(contentDocument.defaultView.top.document, "codeSites", []).push(locationSite);
         
-        
+        let scriptElement;
+        if (aContentType === 2) { // "real" JavaScript include
+          if (originSite && !this.isJSEnabled(originSite) &&
+              (aRequestOrigin.schemeIs("http") || aRequestOrigin.schemeIs("https")) &&
+              (aContentLocation.schemeIs("http") || aContentLocation.schemeIs("https"))) {
+            // JavaScript-disabled page with script inclusion
+            this.syncUI(contentDocument);
+            return this.reject("Script inclusion on forbidden page", arguments);
+          }
+          
+          // plugin instantiation hacks
+          if (this.contentBlocker && aRequestOrigin &&
+                (aRequestOrigin.schemeIs("http") ||
+                  aRequestOrigin.schemeIs("https") ||
+                  aRequestOrigin.schemeIs("file"))
+              ) {
+            this.applyPluginPatches(contentDocument);
+          }
+
+          forbid = !(originSite && locationSite == originSite);
+          
+          scriptElement = aContext instanceof Ci.nsIDOMHTMLScriptElement;
+        } else isScript = scriptElement = false;
+
         if (forbid) forbid = !this.isJSEnabled(locationSite);
-        if (forbid && this.ignorePorts && /:\d+$/.test(locationSite)) {
-          forbid = !this.isJSEnabled(locationSite.replace(/:\d+$/, ''));
-        }
-  
-        if ((untrusted || forbid) && aContentLocation.scheme != "data") {
-          if (isScript) ScriptSurrogate.apply(contentDocument, locationURL);
+        if (forbid && this.ignorePorts && /:\d+$/.test(locationSite))
+          forbid = !(this.isJSEnabled(locationSite.replace(/:\d+$/, '')) && this.autoTemp(locationSite));
+
+        if ((untrusted || forbid) && scheme !== "data") {
+          if (scriptElement) {
+            if (ScriptSurrogate.apply(contentDocument, locationURL)) {
+              let surrogates = this.getExpando(contentDocument, "surrogates", {});
+              surrogates[locationURL] = true;
+            }
+          }
+          
+          this.syncUI(contentDocument);
+          
           return this.reject(isScript ? "Script" : "XSLT", arguments);
         } else {
           
-          if (isScript && this.executingJSURL(contentDocument.defaultView.top.document) &&
-              (aContext instanceof CI.nsIDOMHTMLScriptElement) &&
-              !this.jsPolicySites.matches(this.getSite(contentDocument.defaultView.location.href))) {
-            this.bookmarkletImport(aContext, locationURL);
-            return this.reject("Bookmarklet inclusion, already imported synchronously", arguments);
+          if (scriptElement) {
+            
+            ScriptSurrogate.apply(contentDocument, locationURL, "<");
+            
+            if (this.executingJSURL(contentDocument.defaultView.top.document) &&
+                !this.jsPolicySites.matches(this.getSite(contentDocument.defaultView.location.href))) {
+              this.bookmarkletImport(aContext, locationURL);
+              return this.reject("Bookmarklet inclusion, already imported synchronously", arguments);
+            }
+            
           }
           
           return CP_OK;
@@ -509,25 +554,25 @@ const MainContentPolicy = {
               if (isSilverlight) {
                 if (logIntercept) this.dump("Silverlight " + aContentLocation.spec + " " + typeof(aContext) + " " + aContentType + ", " + aInternalCall);
                
-                
-                forbid = aContentType == 12 || !this.POLICY_OBJSUB;
                 this.setExpando(aContext, "silverlight", aContentType != 12);
                 if (!forbid) return CP_OK;
                 
                 locationURL = this.resolveSilverlightURL(aRequestOrigin, aContext);
                 locationSite = this.getSite(locationURL);
                 originURL = aRequestOrigin && aRequestOrigin.spec;
-                if (!this.POLICY_OBJSUB)  forbid = locationURL != originURL;
                 
                 if(!forbid || this.isAllowedObject(locationURL, mimeKey, locationSite) ||
-                   this.isAllowedObjectById(aContext.id, locationURL, originURL, mimeKey, locationSite)) {
+                   this.isAllowedObjectByDOM(aContext, locationURL, originURL, mimeKey, locationSite)) {
                   if (logIntercept && forbid) this.dump("Silverlight " + locationURL + " is whitelisted, ALLOW");
                   return CP_OK;
                 }
               } else if (isFlash) {
                 locationURL = this.addFlashVars(locationURL, aContext);
               }
-            } else if (!(this.allowedMimeRegExp && this.allowedMimeRegExp.test(aMimeTypeGuess))) {
+            } else if (!(this.allowedMimeRegExp &&
+                          (this.allowedMimeRegExp.test(aMimeTypeGuess) ||
+                            this.allowedMimeRegExp.test(aMimeTypeGuess + "@" + locationSite))
+                      )) {
               forbid = this.forbidPlugins && !(isJava || isFlash || isSilverlight);
               if (forbid) {
                 locationURL = this.addObjectParams(locationURL, aContext);
@@ -536,24 +581,25 @@ const MainContentPolicy = {
           }
         }
       }
-  
-      if(forbid && !this.contentBlocker) {
+
+      if (forbid && !this.contentBlocker) {
         
         originURL = originURL || (aRequestOrigin && aRequestOrigin.spec);
         originSite = originSite || this.getSite(originURL);
-      
-        var originOK = originSite 
-          ? this.isJSEnabled(originSite) 
-          : /^(?:javascript|data):/.test(originURL); // if we've got such an origin, parent should be trusted
         
-        var locationOK = locationSite 
+        let jsRx = /^(?:javascript|data):/;
+        
+        let originOK = originSite 
+          ? this.isJSEnabled(originSite) 
+          : jsRx.test(originURL); // if we've got such an origin, parent should be trusted
+        
+        let locationOK = locationSite 
               ? this.isJSEnabled(locationSite) 
-              : // use origin for javascript: or data:
-                /^(?:javascript|data):/.test(locationURL) && originOK
+              : jsRx.test(locationURL) && originOK // use origin for javascript: or data:
         ;
         
-        if (!locationOK && locationSite && this.ignorePorts && /:\d+$/.test(locationSite)) {
-          if (this.isJSEnabled(locationSite.replace(/:\d+$/, ''))) {
+        if (!locationOK && locationSite && this.ignorePorts && this.portRx.test(locationSite)) {
+          if (this.isJSEnabled(locationSite.replace(this.portRx, ''))) {
             locationOK = this.autoTemp(locationSite);
           }
         }
@@ -565,27 +611,23 @@ const MainContentPolicy = {
           ));
       }
        
-      if (/\binnerHTML\b/.test(new Error().stack)) {
-        if (this._bug453825) {
-          aContext.ownerDocument.location.href = 'javascript:window.__defineGetter__("top", (Window.prototype || window).__lookupGetter__("top"))';
-          if (this.consoleDump) this.dump("Locked window.top (bug 453825 work-around)");
-        }
-        if (this._bug472495) {
-          aContext.ownerDocument.defaultView.addEventListener("DOMNodeRemoved", this._domNodeRemoved, true);
-          if (this.consoleDump) this.dump("Added DOMNodeRemoved (bug 472495 work-around)");
-        }
-      }
+      mustCountObject = true;
       
-      
-      this.delayExec(this.countObject, 0, aContext, locationSite); 
-      
-      forbid = forbid && !(/^file:\/\/\//.test(locationURL) && /^resource:/.test(originURL || (aRequestOrigin && aRequestOrigin.spec || ""))); // fire.fm work around
+      if (forbid) forbid = !(aContentLocation.schemeIs("file") && aRequestOrigin && aRequestOrigin.schemeIs("resource")); // fire.fm work around
       
       if (forbid) {
+        
+        if (!originSite) originSite = this.getSite(originURL || aRequestOrigin && aRequestOrigin.spec || "");
+        
+        if (isJava && originSite && /^data:application\/x-java\b/.test(locationURL) ||
+            aContentType === 15 && locationURL === "data:,") {
+          locationURL = locationSite = originSite;
+        }
+        
         try {  // moved here because of http://forums.mozillazine.org/viewtopic.php?p=3173367#3173367
           if (this.getExpando(aContext, "allowed") || 
-            this.isAllowedObject(locationURL, mimeKey, locationSite) ||
-            this.isAllowedObjectById(aContext.id, locationURL, originURL, mimeKey, locationSite)
+            this.isAllowedObject(locationURL, mimeKey, locationSite, originSite) ||
+            this.isAllowedObjectByDOM(aContext, locationURL, originURL, mimeKey, locationSite, originSite)
             ) {
             this.setExpando(aContext, "allowed", true);
             return CP_OK; // forceAllow
@@ -599,14 +641,14 @@ const MainContentPolicy = {
             return this.reject("Deferred Legacy Frame " + locationURL, arguments);
         } else {
           try {
-            if ((aContentType == 5 || aContentType == 7 || aContentType == 12 || aContentType == 14 || aContentType == 15) && (aContext instanceof CI.nsIDOMNode)) {
+            if ((aContentType == 5 || aContentType == 7 || aContentType == 12 || aContentType == 14 || aContentType == 15) && (aContext instanceof Ci.nsIDOMNode)) {
               if (locationURL != "data:application/x-noscript-blocked,") {
-                if (this.consoleDump & LOG_CONTENT_BLOCK)
-                  this.dump("tagForReplacement");
-
+                mustCountObject = false; // we do it in _preProcessObjectElements()
                 this.delayExec(this.tagForReplacement, 0, aContext, {
                   url: locationURL,
-                  mime: mimeKey
+                  site: locationSite,
+                  mime: mimeKey,
+                  originSite: originSite
                 });
               }
             } else if (this.consoleDump & LOG_CONTENT_BLOCK) this.dump("Context is not a DOMNode? " + aContentType);
@@ -630,13 +672,13 @@ const MainContentPolicy = {
       return this.reject("Content (Fatal Error, " + e  + " - " + e.stack + ")", arguments);
     } finally {
       
+      if (mustCountObject) this.delayExec(this.countObject, 0, aContext, locationSite);
+      
       if (!aInternalCall) PolicyState.removeCheck(aContentLocation);
       
       if (isHTTP) PolicyState.save(unwrappedLocation, arguments);
       else PolicyState.reset();
       
-      
-      
     }
     return CP_OK;
   },
@@ -652,6 +694,6 @@ const MainContentPolicy = {
 
 var PolicyUtil = {
   isXSL: function(ctx) {
-    return ctx && !(ctx instanceof CI.nsIDOMHTMLLinkElement || ctx instanceof CI.nsIDOMHTMLStyleElement);
+    return ctx && !(ctx instanceof Ci.nsIDOMHTMLLinkElement || ctx instanceof Ci.nsIDOMHTMLStyleElement || ctx instanceof Ci.nsIDOMHTMLDocument);
   }
-}
\ No newline at end of file
+};
