@@ -1,26 +1,5 @@
-/***** BEGIN LICENSE BLOCK *****
-
-NoScript - a Firefox extension for whitelist driven safe JavaScript execution
-Copyright (C) 2004-2009 Giorgio Maone - g.maone@informaction.com
-
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-***** END LICENSE BLOCK *****/
-
-const CC = Components.classes;
-const CI = Components.interfaces;
+const Cc = Components.classes;
+const Ci = Components.interfaces;
 
 var ns = noscriptUtil.service;
 
@@ -42,9 +21,10 @@ var nsopt = {
     this.utils.resumeTabSelections();
     
     abeOpts.init();
-     
+    EF.init();
+    
     var locked = ns.locked;
-    for each (widget in ["urlText","urlList", "jsglobal", "addButton", "removeButton", "importButton", "exportButton"]) {
+    for each (var widget in ["urlText","urlList", "jsglobal", "addButton", "removeButton", "importButton", "exportButton"]) {
       this[widget] = $(widget);
       if(locked) this[widget].disabled = true;
     }
@@ -91,7 +71,7 @@ var nsopt = {
     
     this.soundChooser.setSample(ns.getPref("sound.block"));
     
-    this.autoAllowGroup = new ConditionalGroup(ns, "autoAllow", 0);
+    this.autoAllowGroup = new ConditionalGroup(ns, "autoAllow", 1);
     this.toggleGroup = new ConditionalGroup(ns, "toolbarToggle", 3);
     
     var val = ns.getPref("allowHttpsOnly", 0);
@@ -111,59 +91,51 @@ var nsopt = {
         "xssEx", 
         ns.rxParsers.multi,
         ns.getPref("filterXExceptions"));
-    this.jarEx = new  RegExpController(
-        "jarEx", 
-        ns.rxParsers.multi,
-        ns.getPref("forbidJarDocumentsExceptions"));
     
     // hide incompatible options
-    if(top.opener && top.opener.noscriptOverlay && !top.opener.noscriptOverlay.getNotificationBox()) {
-      // Moz/SeaMonkey, no notifications
-      $("fx-notifications").setAttribute("hidden", "true");
+    let browserWin = ns.dom.mostRecentBrowserWindow;
+    
+    if (browserWin) {
+      if (!browserWin.document.getElementById("noscript-statusIcon"))
+        $("opt-statusIcon").setAttribute("hidden", "true");
+    
+      if(browserWin.noscriptOverlay && !browserWin.noscriptOverlay.getNotificationBox())
+        $("fx-notifications").setAttribute("hidden", "true");
     }
     
-    ["clearClick", "opacizeObject"].forEach(function(c) {
+    ["clearClick"].forEach(function(c) {
       var pref = ns.getPref(c);
       Array.forEach($(c + "Opts").getElementsByTagName("checkbox"), function(cbx) {        
         cbx.setAttribute("checked", !(pref & parseInt(cbx.getAttribute("value"))) ? "false" : "true");
       });
     });
+       
     
-    if (!ns.clearClickHandler.isSupported(document))
-      ["clearClickOpts", "opt-clearClick.prompt"].forEach(function(id) {
-        $(id).setAttribute("hidden", "true"); // Fx <= 1.5
-      });
-    
-    
-    if (!ns.placesSupported) {
+    if (!ns.placesSupported)
       $("opt-placesPrefs").setAttribute("hidden", "true");
-    }
     
-    if (ns.canSerializeConf) this.initSerializeButtons();
-    
-    if (ns.smUninstaller) this.initUninstallButton();
-    
-    // $("policy-tree").view = policyModel;
-    window.sizeToContent();
+    this.initExtraButtons();
     
     this.addButton.setAttribute("enabled", "false");
     this.removeButton.setAttribute("enabled", "false");
+    
+    this.toggleHoverUI();
+    
+    
+
+    window.sizeToContent();
   },
   
-  initUninstallButton: function() {
-    this.utils.moveButtonsDown("uninstallButton");
-  },
-  
-  uninstall: function() {
-    ns.smUninstaller.appUninstall(window);
+
+  initExtraButtons: function() {
+    this.utils.moveButtonsDown("donateButton", "", "importConfButton", "exportConfButton");
   },
   
   
-  initSerializeButtons: function() {
-    this.utils.moveButtonsDown("importConfButton", "exportConfButton");
+  donate: function() {
+    noscriptUtil.openDonate("options");
   },
   
-  
   importConf: function() {
     this.chooseFile(
       this.buttonToTitle("importConfButton"),
@@ -207,6 +179,20 @@ var nsopt = {
   },
   
   save: function() {
+    /*
+    if (!$("abeRuleset-errors").hidden) {
+      let p = noscriptUtil.prompter;
+      if (p.confirmEx(window,
+          noscriptUtil.getString("ABE.syntaxError"),
+          $("abeRuleset-errors").value,
+          p.BUTTON_TITLE_SAVE * p.BUTTON_POS_0 +
+            p.BUTTON_TITLE_DONT_SAVE * p.BUTTON_POS_1 +
+            p.BUTTON_POS_1_DEFAULT,
+          null, null, null, null, {}) === 1
+        ) 
+      return false;
+    }
+    */
     this.utils.visitCheckboxes(
       function(prefName, inverse, checkbox, mozilla) {
         if(checkbox.getAttribute("collapsed")!="true") {
@@ -235,7 +221,7 @@ var nsopt = {
       }
     });
     
-    ["clearClick", "opacizeObject"].forEach(function(c) {
+    ["clearClick"].forEach(function(c) {
       var pref = 0;
       Array.forEach($(c + "Opts").getElementsByTagName("checkbox"), function(cbx) {
         if (cbx.checked) pref = pref | parseInt(cbx.getAttribute("value"));
@@ -250,21 +236,23 @@ var nsopt = {
     ns.setPref("sound.block", this.soundChooser.getSample());
     
     this.autoAllowGroup.persist();
-    this.toggleGroup.persist();
+    
+    if (!(ns.getPref("hoverUI.excludeToggling") && $("opt-hoverUI").checked)) {
+      this.toggleGroup.persist();
+    }
     
     ns.setPref("allowHttpsOnly", $("sel-allowHttpsOnly").selectedIndex);
     
     var exVal = this.xssEx.getValue();
     if(this.xssEx.validate() || !/\S/.test(exVal)) 
       ns.setPref("filterXExceptions", exVal);
-    var exVal = this.jarEx.getValue();
-    if(this.jarEx.validate() || !/\S/.test(exVal)) 
-      ns.setPref("forbidJarDocumentsExceptions", exVal);
     
     if (this.tempRevoked) {
       ns.resetAllowedObjects();
     }
     
+    EF.save();
+    
     var global = this.jsglobal.getAttribute("checked") == "true";
     var untrustedSites = this.untrustedSites;
     var trustedSites = this.trustedSites;
@@ -285,8 +273,9 @@ var nsopt = {
       }
       ns.jsEnabled = global;
     });
+    return true;
   },
-
+  
   urlListChanged: function() {
     const selectedItems = this.urlList.selectedItems;
     var removeDisabled = true;
@@ -303,6 +292,21 @@ var nsopt = {
     this.urlChanged();
   },
   
+  openInfo: function(ev) {
+    if (ev.button === 1) {
+      setTimeout(function() {
+        const selectedItems = nsopt.urlList.selectedItems;
+        const domains = [];
+        for (let j = selectedItems.length; j-- > 0;) {
+          let site = selectedItems[j].value;
+          let d = site.indexOf(":/") > 0 ? ns.getDomain(site) : site;
+          if (d && domains.indexOf(d) === -1) domains.push(d);
+        }
+        domains.forEach(noscriptUtil.openInfo, noscriptUtil);
+      }, 0); // delayed to let middle-click autoselect the underlying item
+    }
+  },
+  
   urlChanged: function() {
     var url = this.urlText.value;
     if(url.match(/\s/)) url = this.urlText.value = url.replace(/\s/g,'');
@@ -404,8 +408,7 @@ var nsopt = {
       }
       return;
     }
-    
-    // TODO: hide flickering
+
     this.populateUrlList();
     try {
       var rowCount = ul.getRowCount();
@@ -444,14 +447,14 @@ var nsopt = {
   
   chooseFile: function(title, mode, callback) {
     try {
-     const IFP = CI.nsIFilePicker;
-      const fp = CC["@mozilla.org/filepicker;1"].createInstance(IFP);
+      const IFP = Ci.nsIFilePicker;
+      const fp = Cc["@mozilla.org/filepicker;1"].createInstance(IFP);
       
-      fp.init(window,title, IFP["mode" + mode]);
-      fp.appendFilters(IFP.filterText);
-      fp.appendFilters(IFP.filterAll);
-      fp.filterIndex = 0;
-      fp.defaultExtension = ".txt";
+      fp.init(window, title, IFP["mode" + mode]);
+      fp.displayDirectory = Cc["@mozilla.org/file/directory_service;1"]
+                              .getService(Ci.nsIDirectoryServiceProvider)
+                              .getFile("Home",{});
+      fp.defaultExtension = "txt";
       const ret = fp.show();
       if(ret == IFP.returnOK || 
           ret == IFP.returnReplace) {
@@ -507,58 +510,112 @@ var nsopt = {
   
   buttonToTitle: function(btid) {
     return "NoScript - " + $(btid).getAttribute("label");
+  },
+  
+  toggleHoverUI: function(cbx) {
+    if (ns.getPref("hoverUI.excludeToggling")) {
+      let cbx = $("cbx-toolbarToggle");
+      if ($("opt-hoverUI").checked) {
+        if (!cbx.disabled) {
+          this._savedToolbarToggleStatus = cbx.checked;
+          cbx.disabled = true;
+          cbx.checked = false;
+          this.toggleGroup.changed();
+        }
+      } else {
+        if (cbx.disabled) {
+          cbx.disabled = false;
+          cbx.checked = this._savedToolbarToggleStatus;
+        }
+      }
+    }
   }
-
+  
 }
 
-var ABE = ns.__parent__.ABE;
+var ABE = ns.ABE;
 
 var abeOpts = {
   selectedRS: null,
-  _map: {},
+  _map: {__proto__: null},
   errors: false,
+  QueryInterface: ns.wan.QueryInterface, // dirty hack, we share the same observer ifaces
+  
   init: function() {
-    
-    if (!(ABE.legacySupport || ABE.__parent__.Thread.canSpin)) {
-      var tab = $("nsopt-tabABE");
-      if (tab.selected) {
-        tab.parentNode.selectedIndex = 0;
-      }
-      tab.hidden = true;
-      return;
-    }
-    
     this.list = $("abeRulesets-list");
-    ABE.updateRules();
     this.populate();
-    window.addEventListener("focus", function(ev) {
-      if (ABE.updateRulesNow()) {
-        abeOpts.populate();
-        if (abeOpts.errors) {
-          abeOpts.errors = false;
-          noscriptUtil.prompter.alert(window, "ABE", ns.getString("ABE.syntaxError"));
+    
+    this.updateWAN(ns.wan.ip);
+    const OS = ns.os;
+    OS.addObserver(this, ns.wan.IP_CHANGE_TOPIC, true);
+    OS.addObserver(this, ABE.RULES_CHANGED_TOPIC, true);
+  },
+  
+  observe: function(subject, topic, data) {
+    if (topic === ns.wan.IP_CHANGE_TOPIC) this.updateWAN(data);
+    else if (topic === ABE.RULES_CHANGED_TOPIC) {
+      this.populate();
+      this.errors = false;
+    }
+  },
+  
+  updateWAN: function(ip) {
+    $("opt-ABE.wanIpAsLocal").label = ns.getString("ABE.wanIpAsLocal", [ip || "???"]);
+  },
+  
+  reset: function() {
+    ABE.resetDefaults();
+  },
+  input: function() {
+    abeOpts.dirty = abeOpts.list.selectedItem;
+  },
+  changed: function(i) {
+    let current = i || this.list.selectedItem;
+   
+    if (current && this.dirty) {
+      
+      let name = current.value;
+      let source = $("abeRuleset-text").value;
+      let ruleset = ABE.createRuleset(name, source);
+      if (ruleset.errors && this.dirty) {
+        this.dirty = null;
+        let p = noscriptUtil.prompter;
+        if (p.confirmEx(window,
+            noscriptUtil.getString("ABE.syntaxError"),
+            ruleset.errors.join("\n"),
+            p.BUTTON_TITLE_SAVE * p.BUTTON_POS_0 +
+              p.BUTTON_TITLE_DONT_SAVE * p.BUTTON_POS_1 +
+              p.BUTTON_POS_1_DEFAULT,
+            null, null, null, null, {}) === 1
+          ) {
+          this.sync();
+          return;
         }
       }
-    }, false);
+      this.dirty = null;
+      ABE.storeRuleset(name, source);
+    }
   },
   
   _populating: false,
   populate: function() {
+    if (this._populating) return;
     this._populating = true;
     this.errors = false;
+    this.dirty = null;
     try {
-      this._map = {};
+      const map = {__proto__: null};
       var l = this.list;
       for(var j = l.getRowCount(); j-- > 0; l.removeItemAt(j));
       var rulesets = ABE.rulesets;
       var selItem = null;
       if (rulesets) {
-        var sel = this.selectedRS && this.selectedRS.name;
+        var sel = this.selectedRS && this.selectedRS.name || "USER";
         this.selectedRS = null;
         var i, name;
         for each (var rs in rulesets) {
           name = rs.name;
-          this._map[name] = rs;
+          map[name] = rs;
           i = l.appendItem(name, name);
           if (rs.disabled) i.setAttribute("disabled", "true");
           if (sel == name) selItem = i;
@@ -568,6 +625,7 @@ var abeOpts = {
           }
         }
       }
+      this._map = map; 
       l.selectedItem = selItem;
       this.sync();
     } finally {
@@ -576,11 +634,17 @@ var abeOpts = {
   },
   
   selected: function(i) {
+    if (this.dirty) {
+      let selIndex = this.list.selectedIndex;
+      this.changed(this.dirty);
+      this.list.selectedIndex = selIndex;
+    }
     if (!this._populating) this.sync();
+    this.dirty = null;
   },
   
   select: function(rs) {
-    var name = rs && rs.name;
+    var name = rs && rs.name || rs;
     if (!name) return;
     var l = this.list;
     if (l.selectedItem && l.selectedItem.value == name) return;
@@ -605,18 +669,26 @@ var abeOpts = {
     }
     
     $("abeEnable-button").disabled = ! ($("abeDisable-button").disabled = !rs || rs.disabled);
-    $("abeEdit-button").disabled = !rs || rs.site;
     $("abeRefresh-button").disabled = this.list.getRowCount() == 0;
     
     var text = $("abeRuleset-text");
     text.className = selItem && selItem.className || '';
     text.disabled = !selItem || selItem.disabled;
-    text.value = rs && (rs.errors && rs.errors.join("\n\n") || rs.source) || '';
+    text.value = rs && rs.source;
+    
+    text = $("abeRuleset-errors");
+    if (rs && rs.errors) {
+      text.hidden = false;
+      text.value = rs.errors.join("\n");
+    }
+    else {
+      text.hidden = true;
+      text.value = "";
+    }
   },
   
   refresh: function() {
     ABE.refresh();
-    this.populate();
   },
   
   toggle: function(enabled) {
@@ -630,64 +702,172 @@ var abeOpts = {
     }
     ns.setPref("ABE.disabledRulesetNames", ABE.disabledRulesetNames);
     this.sync();
-  },
-  
-  edit: function(i) {
-    i = i || this.list.selectedItem;
-    if (!i) return;
-    var file = ABE.getRulesetFile(i.value);
-    if (!(file instanceof CI.nsILocalFile)) return;
-   
-    try {
-      file.launch();
-      return;
-    } catch(e) {
-      // probably a *X platform...
-    }
-    
-    var ed = this.editor;
-    if (!ed) return;
-
-    var mimeInfoService = CC["@mozilla.org/uriloader/external-helper-app-service;1"]
-        .getService(CI.nsIMIMEService);
-    var mimeInfo = mimeInfoService
-      .getFromTypeAndExtension( "application/x-abe-rules", "abe" );
-    mimeInfo.preferredAction = mimeInfo.useHelperApp;
-    
-    if ("nsILocalHandlerApp" in CI) {
-      var handler =  CC["@mozilla.org/uriloader/local-handler-app;1"].createInstance(CI.nsILocalHandlerApp);
-      handler.executable = ed;
-      ed = handler;
-    }
-    mimeInfo.preferredApplicationHandler = ed;
-    mimeInfo.launchWithFile(file);      
-  
-  },
-  
-  get editor() {
-    var ed = null;
-    try {
-      ed = ns.prefs.getComplexValue("abe.editor", CI.nsILocalFile);
-      ed.followLinks = true;
-      if (ed.exists() && ed.isExecutable()) return ed;
-      ed = null;
-    } catch(e) {}
-    const IFP = CI.nsIFilePicker;
-    const fp = CC["@mozilla.org/filepicker;1"].createInstance(IFP);
-      
-    fp.init(window, ns.getString("abe.chooseEditor"), IFP.modeOpen);
-    fp.appendFilters(IFP.filterApps);
-    fp.filterIndex = 0;
-    const ret = fp.show();
-    if (ret == IFP.returnOK) {
-      ed = fp.file;
-      if (ed.exists() && ed.isExecutable()) {  
-        ns.prefs.setComplexValue("abe.editor", CI.nsILocalFile, ed);
-      } else ed = null;
-    }
-    return ed;
   }
   
 }
 
-
+var EF = {
+  _dirty: false,
+  _compare: function(a, b) { return a.name > b.name ? 1 : a.name < b.name ? -1 : 0; },
+  currentFilter: null,
+  get list() {
+    delete this.list;
+    return this.list = $("ef-list");
+  },
+  get filters() {
+    delete this.filters;
+    return this.filters = ns.externalFilters.cloneFilters();
+  },
+  
+  init: function() {
+    var tab = $("nsopt-tabEF");
+    if (!ns.externalFilters.supported) {
+      if (tab.selected) {
+        tab.parentNode.selectedIndex = 0;
+      }
+      tab.hidden = true;
+      return;
+    }
+    
+    if (tab.selected) {
+      this.populate();
+    } else {
+      tab.addEventListener("command", function(ev) {
+        tab.removeEventListener("command", arguments.callee, false);
+        EF.populate();
+      }, false);
+    }
+  },
+  
+  populate: function() {
+    const ef = ns.externalFilters;
+    const list = this.list;
+    const filters = this.filters;
+    filters.sort(this._compare);
+    list.removeAllItems();
+    list.selectedItem = null;
+    
+    const filterName = ef.lastFilterName || filters[0] && filters[0].name; 
+    
+    filters.forEach(function(f) {
+      var mi = list.appendItem(f.name);
+      if (!f.valid) mi.setAttribute("class", "noscript-error");
+      if (f.name == filterName) {
+        list.selectedItem = mi;
+        $("ef-exe").value = f.exe && f.exe.path || '';
+        $("ef-type").value = f.contentType;
+        $("ef-exceptions").value = f.whitelist && f.whitelist.source || '';
+        this.currentFilter = f;
+      }
+    }, this);
+    
+    ["ef-remove", "ef-browse", "ef-type", "ef-exceptions"].map($).forEach(
+      this.currentFilter
+      ? function(el) { el && el.removeAttribute("disabled"); }
+      : function(el) { el && el.setAttribute("disabled", "true"); }
+    );
+    
+    if (this.currentFilter) {
+      if (this.currentFilter.builtIn) {
+        $("ef-remove").setAttribute("disabled", "true");
+      } else {
+        $("ef-remove").removeAttribute("disabled");
+      }
+    }
+    this.validate();
+  },
+  
+  onSelect: function(ev) {
+    ns.externalFilters.lastFilterName = this.list.selectedItem && this.list.selectedItem.label;
+    this.populate();
+  },
+  
+  onTypeChange: function(ev) {
+    if (!this.currentFilter) return;
+    this.currentFilter.contentType = ev.target.value;
+    this._dirty = true;
+  },
+  
+  onExceptionsChange: function(ev) {
+    if (this.currentFilter) {
+      var node = ev.target;
+      var wl = new ns.AddressMatcher(node.value);
+      if (wl.rx || !node.value) {
+        node.removeAttribute("class");
+        this.currentFilter.whitelist = wl;
+        this._dirty = true;
+      } else {
+        node.setAttribute("class", "noscript-error");
+      }
+    }
+  },
+  
+  locateExe: function(f) {
+    f = f || this.currentFilter;
+    if (!f) return;
+    
+    const IFP = Ci.nsIFilePicker;
+    const fp = Cc["@mozilla.org/filepicker;1"].createInstance(IFP);
+      
+    fp.init(window, ns.getString("ef.locateExe", [f.name]), IFP.modeOpen);
+    fp.appendFilters(IFP.filterApps);
+    fp.filterIndex = 0;
+    const ret = fp.show();
+    if (ret == IFP.returnOK) {
+      var exe = fp.file;
+      if (exe.exists() && exe.isExecutable()) {  
+        f.exe = exe;
+        $("ef-exe").value = exe.path;
+        this._dirty = true;
+      }
+    }
+    this.validate();
+  },
+  
+  validate: function() {
+    var mi = this.list.selectedItem;
+    if (mi) {
+      if (this.currentFilter && !this.currentFilter.valid && this.list.selectedItem) {
+        mi.setAttribute("class", "noscript-error");
+      } else {
+        mi.removeAttribute("class");
+        return true;
+      }
+    }
+    return false;
+  },
+  
+  create: function() {
+    const ret = { value: "" };
+    if(noscriptUtil.prompter.prompt(window, document.title,
+          ns.getString("ef.newName"), ret, null, {}) && 
+       /^[a-z]/i.test(ret.value)
+       ) {
+      
+      const ef = ns.externalFilters;
+      var f = ef.create(ret.value);
+      if (!f.name) return;
+      
+      if (!this.filters.some(f.same, f)) {
+        this.locateExe(f);
+        if (!f.exe) return;
+        this.filters.push(f);
+      }
+      
+      ef.lastFilterName = f.name;
+      this.populate();
+    }
+  },
+  
+  remove: function() {
+    if (this.currentFilter &&
+        noscriptUtil.prompter.confirm(window, document.title, ns.getString("confirm"))) {
+      this.filters.splice(this.filter.indexOf(this.currentFilter), 1);
+      this.populate();
+    }
+  },
+  
+  save: function() {
+    if (this._dirty) ns.externalFilters.save(this.filters);
+  }
+}
